%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Reading External DB  including effective vars and preparation of observed data %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% housekeeping
clear all; close all; clc

addpath('data');
addpath('results');
addpath('utils');

disp('Reading model and data...')

% loading model and database
load model.mat;

dm = dbload('cz_monthly.csv');
dq = dbload('cz_quarterly.csv');
dy = dbload('cz_yearly.csv');
de = dbload('cz_baseline_tunes.csv');

%% Block of date definitions
% historical date for 
starthist = qq(1995,1);

% End of history
endhist = qq(2016,3);
% % End of external sector projection
% endfcast = endhist + 40;

%% switchers for reporting

doDataReport = true;
doNtfReport  = false;

%% Graphs
sty = struct();
sty.line.linewidth           = 3;
sty.legend.fontsize          = 14;
sty.title.fontsize           = 14;
sty.axes.fontsize            = 14;
%sty.legend.orientation       = ''

barstyle                     = sty;
barstyle.legend.orientation  = 'horizontal';
barstyle.legend.location     = 'northoutside';

set(0,'defaulttextfontweight','normal');
set(0,'defaultaxesfontweight','normal');

%% Start report for showing Czech data

if doDataReport
disp('Data Report...')

% run seasonal adjustment for all series with suffix "_su" in the name,
% save the series under name without the suffix
dm     = dbbatch(dm,'$1','x12(dm.$0)','nameFilter=','(.*)_su');
dq     = dbbatch(dq,'$1','x12(dq.$0)','nameFilter=','(.*)_su');
list_12 = {'cpi_infl', 'net_infl_cont', 'regul_infl_cont', 'food_infl_cont', 'infl_xoil_cont' 'infl_energy_cont'};
dm = dbbatch(dm,'mom_$0','(dm.$0/dm.$0{-1})^12*100-100','namelist',list_12);
dm = dbbatch(dm,'qoq_$0','(dm.$0/dm.$0{-3})^4*100-100','namelist',list_12);
dm = dbbatch(dm,'yoy_$0','(dm.$0/dm.$0{-12})*100-100','namelist',list_12);


 % combine together net inflation and monetary-policy relevant inflation
 % (the latter has shorter history, but the two are quite similar)
 tmp.mom_cpi_net_su = [dm.infl_net_su; dm.infl_mp_su];
      % form SU index
 tmp.cpi_net_su = tseries(get(tmp.mom_cpi_net_su,'first')-1, 100);
 for dat=get(tmp.mom_cpi_net_su,'range')
     tmp.cpi_net_su(dat) = tmp.cpi_net_su(dat-1)*(1+tmp.mom_cpi_net_su(dat)/100);
 end
 dm.cpi_core = x12(tmp.cpi_net_su);

x = report.new();

% GDP I. 
dq.l_gdp_sa = 100*log(dq.gdp_10_sa);
x.figure('','subplot',[2 1],'style',sty,'legendlocation=','northwest','figurescale',0.7);
x.graph('Realne HDP (100*log, H-P filtr)');
x.series('', [dq.l_gdp_sa hpf(dq.l_gdp_sa)], 'legend',{'Realne HDP','Trend realneho HDP (HP filtr)'});
x.graph('Mezera vstupu (%)');
x.series('', [dq.l_gdp_sa-hpf(dq.l_gdp_sa)], 'legend',{'Gap'});

% GDP II. 
x.figure('Rust realneho HDP (sezone ocisteno, %)', 'style',sty,'legend',true,'legendLocation=','best');
x.graph('');
x.series('Mezirocni'     , pct(dq.gdp_10_sa,-4));
x.series('Mezictvrtletni', (dq.gdp_10_sa/dq.gdp_10_sa{-1})^4*100-100);

% GDP III.
x.figure('', 'style',sty,'legend',true,...
    'legendLocation=', 'bottom', 'figureTrim',[6 6 6 6],'range',qq(2005,1):get(dq.gdp_10_sa,'last'));
dq.net_export = dq.gdp_ex_sa - dq.gdp_im_sa;
dq.gdp_rest = dq.gdp_10_sa - dq.net_export - dq.gdp_cons_hh_sa - dq.gdp_cons_gov_sa - dq.gdp_gfc_sa;
x.graph('Dekompozice rustu realneho HDP (%, mezirocne)');
      x.series('',diff([dq.gdp_cons_hh_sa, dq.gdp_cons_gov_sa, dq.gdp_gfc_sa,...
          dq.net_export, dq.gdp_rest],-4)/dq.gdp_10_sa{-4}*100,...
               'legend',{'Soukroma spotreba','Spotreba vlady','Hruba tvorba kapitalu','Cisty Export','Ostatni'},...
               'plotfunc', @conbar);
      x.series(''     , pct(dq.gdp_10_sa,-4),'legend',NaN);
      x.series('Total', pct(dq.gdp_10_sa,-4));

x.figure('Dekompozice rustu realneho HDP (%, mezictvrtletne)', 'style',sty,'legend',true,...
    'legendLocation=', 'bottom', 'figureTrim',[6 6 6 6],'range',qq(2005,1):get(dq.gdp_10_sa,'last'));
x.graph('');
      x.series('',4*diff([dq.gdp_cons_hh_sa, dq.gdp_cons_gov_sa, dq.gdp_gfc_sa,...
          dq.net_export, dq.gdp_rest],-1)/dq.gdp_10_sa{-1}*100,...
               'legend',{'Soukroma spotreba','Spotreba vlady','Hruba tvorba kapitalu','Cisty Export','Ostatni'},...
               'plotfunc', @conbar);
      x.series(''     , 4*pct(dq.gdp_10_sa,-1),'legend',NaN);
      x.series('Total', 4*pct(dq.gdp_10_sa,-1));
 
% GDP IV. 
dq.l_gdp_e = 100*log(dq.gdp_e);
dq.l_eu28_gdp_e = 100*log(dq.eu28_gdp_e);
dq.l_ea19_gdp_e = 100*log(dq.ea19_gdp_e);

x.figure('HDP (100*log, normovane)', 'style',sty,'legend',true,...
    'legendLocation=','best','range',qq(2004,1):get(dq.gdp_10_sa,'last'));
x.graph('');
x.series('CR',          dq.l_gdp_e      - dq.l_gdp_e(qq(2004,1)));
x.series('EU28',        dq.l_eu28_gdp_e - dq.l_eu28_gdp_e(qq(2004,1)));
x.series('Eurozona 19', dq.l_ea19_gdp_e - dq.l_ea19_gdp_e(qq(2004,1)));

% HDP p.c. vuci eurozone
x.figure('HDP per capita v PPP (index EU28 = 100)', 'style',sty,'legend',true,...
    'legendLocation=','best','figurescale',0.7);
x.graph('');
x.series('CR'           , dy.cz_gdp_index); 
x.series('Eurozona 17'  , dy.ea17_gdp_index);
x.series('EU 28'        , dy.eu28_gdp_index);
  
% RER!!!!
% spocteno z mesicnich hodnot HCPI a nom. kurzu
rng = mm(2000,1):get(dm.cpi_su,'last');
dm.reer_vsea = dm.czk_eur*dm.hicp_ea/dm.cpi_su;

dq.l_reer_vsea = 100*log(dm.reer_vsea);
dq.l_reer_cpi = -100*log(dm.reer_cpi);
 
x.figure('Realny efektivni smenny kurz (normovany, 100*log)','style',sty,'legend',true,...
    'legendLocation=','best','range',rng,'figurescale',0.7);
x.graph('');
x.series('Vuci Eurozone', dq.l_reer_vsea - dq.l_reer_vsea(mm(2000,1)));
x.series('Efektivni (vahy roku 2010)', dq.l_reer_cpi - dq.l_reer_cpi(mm(2000,1)) ); 
 
% Smenny kurz a Outright Forward
rng = mm(2011,1):get(dm.czkeur3m,'last');
x.figure('Smenny kurz a Outright Forward, CZK per EUR','style',sty,'range',rng,...
           'legend',true,'legendlocation=','best','figurescale',0.7);
x.graph();
  x.series('CZK per EUR',dm.czk_eur); % ztucnit
  x.series('OF 3M'  ,dm.czkeur3m);
  x.series('OF 6M'  ,dm.czkeur6m);
  x.series('OF 12M' ,dm.czkeur12m);
  x.series('OF 2Y'  ,dm.czkeur2y);
% x.series('OF 3Y'  ,dm.czkeur3y);
  
% Smenny kurz a Outright Forward
x.figure('Smenny kurz a Outright Forward, CZK per EUR','style',sty,...
         'legend',true,'legendlocation=','northwest','figurescale',0.7);
x.graph();
  x.series('CZKper EUR',dm.czk_eur); % ztucnit
  x.series('OF 3M'  ,dm.czkeur3m);
  x.series('OF 6M'  ,dm.czkeur6m);
  x.series('OF 12M' ,dm.czkeur12m);
  x.series('OF 2Y'  ,dm.czkeur2y);
% x.series('OF 3Y'  ,dm.czkeur3y);
  % vratil jsem zpet i delsi obdobi. Podle me vypovida o tom, jak OF funguji
  
% Obchod
rng = qq(2005,1):get(dq.ca,'last');
fh = figure('visible','off');
%barcon(rng,[dq.exports/dq.gdp_nom -dq.imports/dq.gdp_nom]*100);
% legend('Exporty','Importy');
set(gca,'fontsize',14);
hold on;
plot(rng,[100*dq.ca/dq.gdp_nom_sa  100*(dq.exports-dq.imports)/dq.gdp_nom_sa], 'LineWidth=',2);
grid;
legend('Bezny ucet', 'Obchodni bilance', 'location', 'NW');
hold off;
set(gca,'fontsize',14);
x.userfigure('Zahranicni bilance CR (% HDP)', fh);

fh = figure('visible','off');
barcon(rng,[dq.exports/dq.gdp_nom_sa -dq.imports/dq.gdp_nom_sa]*100);
set(gca,'fontsize',14);
hold on;
plot(rng,(dq.exports/dq.gdp_nom_sa-dq.imports/dq.gdp_nom_sa)*100, 'linewidth', 3, 'color', 'k');
legend('Exporty','Importy', 'Ciste Exporty', 'location', 'NW');
grid
hold off;
set(gca,'fontsize',14);
x.userfigure('Obchodni bilance CR (% HDP)', fh);

% Inflace MoM
fh = figure('visible','off');

barcon(mm(2012,1):get(dm.cpi_infl,'last'),[dm.regul_infl_cont dm.indir_tax_cont,...
    dm.infl_xoil_cont dm.food_infl_cont dm.infl_fuel_cont]);
legend('Regul. ceny','Nepr. dane','Korig. bez paliv a potr.','Potraviny','Paliva',...
    'Location', 'southoutside', 'orientation','horizontal');
set(gca,'fontsize',14);
hold on;
plot(mm(2012,1):get(dm.cpi_infl,'last'),dm.cpi_infl, 'LineWidth=',2); 
hold off;
grid
set(gca,'fontsize',14);
x.userfigure('Struktura inflace (% p.p), MoM', fh);

% Inflace YoY
fh = figure('visible','off');

barcon(mm(2012,1):get(dm.cpi_infl_yoy,'last'),[dm.regul_infl_cont_yoy dm.indir_tax_cont_yoy,...
    dm.infl_xoil_cont_yoy dm.food_infl_cont_yoy dm.infl_fuel_cont_yoy]);
legend('Regul. ceny','Nepr. dane','Korig. bez paliv a potr.','Potraviny','Paliva',...
    'Location', 'southoutside', 'orientation','horizontal');
set(gca,'fontsize',14)
hold on;
grid
plot(mm(2012,1):get(dm.cpi_infl_yoy,'last'),dm.cpi_infl_yoy, 'LineWidth=',2); 
hold off;
set(gca,'fontsize',14)
x.userfigure('Struktura inflace (p.p) YoY', fh);


dq.cpi = convert(dm.cpi,'q','ignoreNaN',true);

% core vs celkova
f = figure('visible','off');
subplot(311)
plot(mm(2011,1):get(dm.cpi_su,'last'), [((dm.cpi/dm.cpi{-1})^12)*100-100 ((dm.cpi_core/dm.cpi_core{-1})^12)*100-100],'linewidth',2);
title('Celkova vs Core inflace, MoM @ar')
grid
subplot(312)
plot(mm(2011,1):get(dm.cpi_su,'last'), [((dm.cpi/dm.cpi{-12}))*100-100 ((dm.cpi_core/dm.cpi_core{-12}))*100-100],'linewidth',2);
title('Celkova vs Core inflace, YoY @ar')
legend('Celkova', 'Core')
grid
subplot(313)
bar(mm(2011,1):get(dm.cpi_su,'last'), [((dm.cpi/dm.cpi{-3})^4)*100-100 ((dm.cpi_core/dm.cpi_core{-3})^4)*100-100]);
title('Celkova vs core, QoQ @ar')
legend('Celkova', 'Core', 'orientation','horizontal');
grid
x.userfigure('',f);

% celkova inflace vs mzdy

f = figure('visible','off');
subplot(211)
plot(qq(2011,1):get(dq.cpi,'last'), [((dq.cpi/dq.cpi{-1})^4)*100-100 ((dq.wage_nom/dq.wage_nom{-1})^4)*100-100],'linewidth',2);
title('Celkova inflace vs nominalni mzdy, QoQ @ar')
legend('Inflace', 'Nom mzdy','Location', 'southoutside', 'orientation','horizontal');
grid
subplot(212)
plot(qq(2011,1):get(dq.wage_nom,'last'), [((dq.wage_nom/dq.wage_nom{-4}))*100-100 dq.wage_real_index-100],'linewidth',2);
title('Nominalni vs realny mzdy, YoY')
legend('Nominalni', 'Realny','Location', 'southoutside', 'orientation','horizontal');
grid
x.userfigure('',f);

% Kurzovy zavazek
sty2 = sty;
sty2.line.linewidth       = {2,2,2,2,2,2,2,2,2,2,2,4};
sty2.line.linestyle       = {'--','--','--','--','--','--','--','--','--','--','--','-'};
nLines                    = 10;
colPalette                = varycolor(nLines);
sty2.line.color           = {};
for iter = 1:nLines
  sty2.line.color{end+1}           = colPalette(iter,:);
end



dq.infheadline = pct(dq.cpi,-4);

list = {'infprog14q01','infprog14q02','infprog14q03','infprog14q04',...
    'infprog15q01','infprog15q02','infprog15q03','infprog15q04',...
    'infprog16q01','infprog16q02','infprog16q03'};

for ind = 1:length(list)
    startPer = get(dq.(list{ind}),'first');
    dq.(list{ind})(startPer-1) = dq.infheadline((startPer-1));
end

rng = qq(2014,1):get(dq.(list{end}),'last');
x.figure('CNB: Prognozy inflace a skutecnost (% mezirocne)', 'style',sty2,...
    'legend',true,'legendLocation=','SE','range',rng,'figurescale',0.7);
x.graph('');
x.series('Prognoza z 2014Q01'     , dq.infprog14q01);
x.series('Prognoza z 2014Q02'     , dq.infprog14q02);
x.series('Prognoza z 2014Q03'     , dq.infprog14q03);
x.series('Prognoza z 2014Q04'     , dq.infprog14q04);
x.series('Prognoza z 2015Q01'     , dq.infprog15q01);
x.series('Prognoza z 2015Q02'     , dq.infprog15q02);
x.series('Prognoza z 2015Q03'     , dq.infprog15q03);
x.series('Prognoza z 2015Q04'     , dq.infprog15q04);
x.series('Prognoza z 2016Q01'     , dq.infprog16q01);
x.series('Prognoza z 2016Q02'     , dq.infprog16q02);
x.series('Prognoza z 2016Q03'     , dq.infprog16q03);
x.series('Headline inflace'       , dq.infheadline);

% Nominalni urokove sazby
rng = mm(2007,1):get(dm.ir_cb_repo,'last');
x.figure('Nominalni urokove sazby (% p.a.)', 'style',sty,'legend',true,'legendLocation=','best','range',rng,'figurescale',0.7);
x.graph('');
x.series('2T Repo'       , dm.ir_cb_repo);
x.series('Pribor 1M'     , 100*dm.int_pribor1m);
x.series('Dluhopisy 2Y'  , 100*dm.int_gb_2y);
x.series('Dluhopisy 5Y'  , 100*dm.int_gb_5y);
x.series('cpi_infl_yoy'  , dm.cpi_infl_yoy);

% Realne urokove sazby
rng = mm(2007,1):get(dm.ir_cb_repo,'last');
x.figure('Realne urokove sazby (ocisteny o spotrebitelskou inflaci, % p.a.)',...
    'style',sty,'legend',true,'legendLocation=','best','range',rng,'figurescale',0.7);
x.graph('');
x.series('2T Repo'       , (dm.ir_cb_repo)     - dm.cpi_infl_yoy);
x.series('Pribor 1M'     , 100*(dm.int_pribor1m) - dm.cpi_infl_yoy);
x.series('Dluhopisy 2Y'  , 100*(dm.int_gb_2y)    - dm.cpi_infl_yoy);
x.series('Dluhopisy 5Y'  , 100*(dm.int_gb_5y)    - dm.cpi_infl_yoy);

% interest rate comparison
libor = dm.i_3m_libor_usd_us*91/100/365;
dm.impRate = ((dm.czk_usd + dm.czk_usd_fwd_3m/10000)*(1+libor)/dm.czk_usd-1)*diag(36500/91);

f = figure('visible', 'off');
p=plot([dm.ir_ib_3m dm.tb_3m dm.ir_cb_repo dm.impRate dm.ir_czeonia],'LineWidth',1.5); 
set(p(1:5),{'linestyle'},{'-';'-';'--';'--';'--'});
grid on
title('Comparison of interest rates');
legend('3M interbank rate - PRIBOR','3M zero coupon T-bill yield curve',...
    'Central bank`s 2W repo rate (RS in model)',...
    '3M Implied Rate plus Premium Based on Covered UIP (percent p.a., CZK/USD Forwards and USD LIBOR Rates)',...
    'CZEONIA reference rate');
x.userfigure('IR section',f);

% Devizove rezervy vybranych bank


dq.cz_gdp_annu_nom = convert(dy.gdp_nom_cz,'q','method','quadsum');
dq.jp_gdp_annu_nom = convert(dy.gdp_nom_jp,'q','method','quadsum');
dq.ch_gdp_annu_nom = convert(dy.gdp_nom_ch,'q','method','quadsum');
x.figure('Devizove rezervy vybranych bank (% HDP)', 'style',sty,'legend',true,'legendLocation=','best','figurescale',0.7);
x.graph('');
x.series('CR'           , 100*(dq.cz_fx/1000)/(4*dq.cz_gdp_annu_nom)); % annulized gdp is used
x.series('Japonsko'     , 100*dq.jp_fx/(4*(dq.jp_gdp_annu_nom)));
x.series('Svycarsko'    , 100*dq.ch_fx/(4*(dq.ch_gdp_annu_nom)));

f = figure('visible','off');
bar(qq(2002,1):qq(2015,3),[dq.gdef_rat dq.ger_gdef_rat dq.fra_gdef_rat]);grid on;
title('Deficit vladnich instituci (sezonne ocisteno, % HDP)');
legend('CR', 'Nemecko', 'Francie', 'location', 'NW');
x.userfigure('',f);

filename = ['results/report_data.pdf'];

%% TOT

% calculating tot based on export and import prices
dm.tot_stat_ind = dm.ex_price_ind/dm.im_price_ind;

f = figure('visible','off');
subplot(211)
p = plot([100*log(dm.tot_stat_ind*100) hpf(100*log(dm.tot_stat_ind*100))],'linewidth',2);grid;
set(p(1),'marker','*');
set(p(2),'linestyle','--');
title('Smenna relace (TOT), prumer 2005, 100*log');
legend('level','HPF trend')
subplot(212)
p = plot([pct(dm.tot_stat_ind,-12)],'linewidth',2,'marker','*');grid;
title('Smenna relace (TOT), YoY');
x.userfigure('',f);

f = figure('visible','off');
subplot(211)
p = plot([100*log(dm.ex_price_ind*100) ],'linewidth',2);grid;
set(p(1),'marker','*');
title('Exportni ceny, prumer 2005, 100*log');
subplot(212)
p = plot(hpf(100*log(dm.im_price_ind*100)),'linewidth',2,'marker','*');grid;
title('Importni ceny, prumer 2005, 100*log');
x.userfigure('',f);

f = figure('visible','off');
subplot(211)
p = plot(100*log(dy.ulc),'linewidth',2);grid;
set(p(1),'marker','*');
title('Unit Labor Cost, 100*log');
subplot(212)
p = plot(diff(100*log(dy.ulc),-1),'linewidth',2,'marker','*');grid;
title('Rust v Unit Labor cost, YoY, pct');
x.userfigure('',f);

    
x.publish(filename,'maketitle',false,'display',false);

end

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%  NTF  using   BVAR    %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%disp('NTF using BVAR ...');

if doNtfReport
addpath('ntf');   % in this directory are: additional functions for reporting NTF, additional data, model for filtering

% Defining the report
rep = report.new('NTF Report');

%% switch of the performance report

% if you want to plot BVAR report set this to true (otherwise false)
BvarperformanceReport = true;

%% setting the range

rngntf_m = 6; % number of months for NTF projection
rngntf_q = 4; % number of quarters for NTF projection

%% transform data (logs, growth rates: mom, yoy)

% monthly for CPI
tmp = struct();
  %=== Core CPI level
      % combine together net inflation and monetary-policy relevant inflation
      % (the latter has shorter history, but the two are quite similar)
      tmp.mom_cpi_net_su = [dm.infl_net_su; dm.infl_mp_su];
      % form SU index
      tmp.cpi_net_su = tseries(get(tmp.mom_cpi_net_su,'first')-1, 100);
      for dat=get(tmp.mom_cpi_net_su,'range')
          tmp.cpi_net_su(dat) = tmp.cpi_net_su(dat-1)*(1+tmp.mom_cpi_net_su(dat)/100);
      end
dm.cpi_net_su = tmp.cpi_net_su;  % just put into dm struct
dm.cpi_net    = x12(dm.cpi_net_su);
dm.cpi        = x12(dm.cpi_su);
dm.usd_eur(mm(2016,2))= 1.109904762;   % update from ogi
dm.czk_eur    = dm.czk_usd*dm.usd_eur;    % ex. rate czk/eur
dm.export     = convert(dq.export,'m');

% monetary aggregates (only yoy growth rates)
% dm.dl_m1     = dm.m1_yoy/;
dm.d12l_m3   = dm.m3_yoy;
dm.d12l_cred = dm.cred_yoy;
dm.oil       = dm.oil_usd*dm.czk_usd;    % oil in CZK

list_m = {'cpi_net','cpi','czk_eur','export','oil','food_ind','czk_usd','m1_yoy'};  % define list of variables for calculating growth rates
% calculate growth rates (m-on-m and y-on-y)
for ind = 1:numel(list_m)
    dm.(['l_' list_m{ind}])    = 100*log(dm.(list_m{ind}));
    dm.(['dl_' list_m{ind}])   = 12*(dm.(['l_' list_m{ind}]) - dm.(['l_' list_m{ind}]){-1});
    dm.(['d12l_' list_m{ind}]) = dm.(['l_' list_m{ind}]) - dm.(['l_' list_m{ind}]){-12};
end

% food adjusted by exchange rate depreciation
dm.dl_food = dm.dl_food_ind + dm.dl_czk_usd;

% quarterly (for GDP)
dq.y_su   = dq.gdp_10_su;       % rename variable for simplicity
dq.y      = x12(dq.y_su);
dq.oil    = exp(dq.L_OIL/100);  % only prediction
dq.food   = exp(dq.L_FOOD/100);
dq.gdp_us = exp(dq.L_GDP_US/100);

dq.gdp_eu_gap = de.OBS_L_GDP_GAP_EU; % external demand
dq.gdp_eu     = dq.eu28_gdp_e;       % alternative -  EU (28)
%dq.gdp_ez     = dq.gdp_10_ez;        % alternative -  EZ (taken from ogi)

list_q = {'y','export','gdp_eu'};

for ind = 1:numel(list_q)
    dq.(['l_' list_q{ind}])   = 100*log(dq.(list_q{ind}));
    dq.(['dl_' list_q{ind}])  = 4*(dq.(['l_' list_q{ind}]) - dq.(['l_' list_q{ind}]){-1});
    dq.(['d4l_' list_q{ind}]) = dq.(['l_' list_q{ind}]) - dq.(['l_' list_q{ind}]){-4};
end

% change of GDP gap (alternative)
dq.dl_gdp_eu_gap =4*(dq.gdp_eu_gap - dq.gdp_eu_gap{-1});

% filter the data                
% gap of interest rates, only monthly data

% Monthly -- for BVAR GDP
 list = {'ir_cb_repo','ir_ib_3m'};

%'----- Hodrick-Prescott: series --> trend and gap components -----
 for ind = 1:numel(list)
     [dm.([list{ind} '_tnd']), dm.([list{ind} '_hpgap'])] = hpf(dm.(list{ind}));
 end

% Kalman filter using for smoothing HP filtered gaps
%----- RWWN filter: gap --> cycle and irregular components -----
%---- Differences of trends, gaps and cycles
m = model('ntf/gap_model.mod','linear',true);
m = solve(m);
m = sstate(m);

db_k = struct;

for ind = 1:numel(list)
    db_k.L_GAP = dm.([list{ind} '_hpgap']);
    [m_f, db_f] = filter(m,db_k,get(db_k.L_GAP,'range'));
    dm.([list{ind} '_gap']) = db_f.mean.l_tnd;
end

% save the database with external variables and variables ready for BVAR
%save('ntfdata.mat')

%% BVAR for CPI

label = 'BVAR: Core CPI';
disp([label,' ...']);

dm.dl_oil_czk = dm.dl_oil{-6};
dm.dl_food_czk = dm.dl_food{-5};

list_rep = {'dl_cpi_net'};                       % define list of variables for BVAR (for report)
list_ext = {'dl_czk_eur','dl_m1_yoy','dl_oil_czk','dl_food_czk'};   % other variables used in BVAR
% export interpolated, but available only up to 2015,12
% zkusit conditionovat promenne, ktere zname
% nepozorovat gapy, radsi sazby

list = [list_rep,list_ext];

dv = struct; % create empty database
dv = dm*list;

smpl     = mm(2008,1):get(dv.dl_cpi_net,'last'); % set estimation range (money agregates starts from 2003,1) 
smplf    = smpl(end)+(1:rngntf_m); % set forecast range
smplfull = smpl(1):smplf(end); % set full range


dummylist = {'dummy1'}; %,'dummy2','dummy3','dummy4','dummy5','dummy6','dummy_l13'}; %define dummy variables

% not necessay
dv.dummy1 = tseries(smplfull,0);
dv.dummy1(mm(2009,1))  = 1;
dv.dummy2 = tseries(smplfull,0);
dv.dummy2(mm(2014,1))  = 1;
dv.dummy_l13 = tseries(smplfull,0);
dv.dummy_l13(mm(2013,1):smplfull(end)) = 1;
dv.dummy3 = tseries(smplfull,0);
dv.dummy3(mm(2005,1))  = 1;
dv.dummy4 = tseries(smplfull,0);
dv.dummy4(mm(2011,1))  = 1;


% order of VAR
p = 4;

% Set up Litterman priors for bayesian estimation of the VAR.

% Litterman priors pull the estimates of all entries in the transition
% matrix down towards zeros, except the diagonal elements in the
% first-order matrix, A(1), which are pulled towards a user-specified
% number, |rho|. In other words, the priors assume that the series have
% independent dynamics, and follow an AR(1) process with autocorrelation
% |rho| -- this also encompasses the cases of multivariate white noise with
% |rho| = 0, and random walks with |rho| = 1.
%
% Litterman's priors are good protection againts overfitting. We use them
% here mainly because the unrestricted (non-bayesian) external VAR shows
% tendency to an unstable model (with one root around or slightly above one
% on a unit circle).
% 
% The priors are implemented through so-called prior dummy observations in
% Iris-toolbox, see Sims and Zha (1998). The |BVAR.litterman| function's input
% arguments are as follows:
%
% # |rho|, i.e. the autocorrelation coefficient towards which the
% first-order diagonal entries will be pulled. We assume |rho = 0| because
% of stationary VAR.
% # The overall weight on the priors, i.e. a number inversely proportional
% to the std errors we associate with the prior distributions. This can be
% a single number or a vector of as many numbers as variables. In the
% latter case, we assign a specific weight on the priors in each individual
% variable's equation. If the variables have very different variances (or
% std deviations) each, it's a good idea to relate the weight to the
% variables' relative std deviations. The |nobs| variable helps us to
% interpret the prior information we supply as a certain number of
% hypothetical artificial observations that give rise to our prior
% parameter values -- note that we need to multiply the variables' std
% devations by the square root of |nobs|.
% # The power factor with which the tightness of the priors increases for
% more distant lags. We set 1, which means all lags are treated with 
% linearly decreasing weight(w = lag^lambda). 
% # The last three input arguments describe the number of variables, the
% order of the level VAR for which these prior will used and a number of
% cointegration vectors.relstd   = nanstd(db2tseries(dv,list));

relstd   = nanstd(db2tseries(dv,list));
nobs     = length(smpl); 
dummyobs = BVAR.litterman(0,sqrt(nobs)*relstd,1);

% Define new Bayesian VAR with exogenous variables
ww = VAR(list,'exogenous',dummylist);   

% Estimate BVAR
[w, data] = estimate(ww, dv, smpl, 'order=',p, 'bvar',dummyobs);

% compute forecast three months ahead
f = forecast(w, dv, smplf, dv, 'meanonly',true, 'dboverlay',true); 

% % adding effect for the near term CPI and components based on expert judgement from the staff 
% listEffects = dbnames(dmfEf);
% 
% for ind = 1:length(listEffects)
%     if ~isempty(dmfEf.(listEffects{ind}))
%         
%         date = get(dmfEf.(listEffects{ind}),'range');
%         
%         for per = 1:numel(date)
%             f.(['dl_cpi_' listEffects{ind}])(date(per)) = f.(['dl_cpi_' listEffects{ind}])(date(per)) +...
%                 (100*log(dmfEf.(listEffects{ind})(date(per))/100+1))/(dm.(['w_cpi_' listEffects{ind}])(date(per))/100);
%             f.dl_cpi(date(per)) = f.dl_cpi(date(per)) + (100*log(dmfEf.(listEffects{ind})(date(per))/100+1));
%         end
%     end
% end


for idx = list_rep  
    % move projection into final database
    id = idx{:};
    dmf.(id) = f.(id);

    % extend level by projected growth
    dmf.(id(2:end)) = dm.(id(2:end));
    for ii = smpl(end)+(1:rngntf_m)
     dmf.(id(2:end))(ii) = dmf.(id(2:end))(ii-1) + dmf.(id)(ii)/12;
    end

    % calculate original seasonally adjusted level
    dmf.(id(4:end)) = exp(dmf.(id(2:end))/100);
 
    % calculate seasonally unadjusted level 
    dmf.([id(4:end),'_sf']) = dmf.(id(4:end))/dm.([id(4:end),'_su']);
    dmf.([id(4:end),'_sf'])(end+(1:12)) = dmf.([id(4:end),'_sf'])(end+(-11:0));
    dmf.([id(4:end),'_su']) = dmf.(id(4:end))/dmf.([id(4:end),'_sf']);
end

% evaluate preformance of the BVAR model
if BvarperformanceReport
rep = report_var_performance(w, dv, smpl, p, rep, label);
end


rep = report_cpi_onepager(dmf, smpl(end), list_rep, rep, label);

% calculate RMSE for alternative specifications
rmse_cpi = sqrt(data.res_dl_cpi_net(:)'*data.res_dl_cpi_net(:));
% disp('RMSE of Core CPI MoM Growth Rate')
% disp([rmse_cpi])

%% BVAR for GDP 

label = 'BVAR: GDP Growth';
disp([label,' ...']);



list_rep = {'dl_y'};    % define list of variables for BVAR
list_ext = {'dl_cpi','ir_ib_3m_gap','dl_gdp_eu'};  %,'unempl_rate'};  % 
% alternatives for external demand   gdp_ez,  gdp_eu, gdp_eu_gap, dl_gdp_eu_gap

list = [list_rep,list_ext];

dv = struct; % create empty database
dv = dq*list_rep;

% other data need to be added into dv structure !
dv.dl_cpi        = convert(dm.dl_cpi,'q');
dv.ir_ib_3m_gap  = convert(dm.ir_ib_3m_gap,'q');
dv.ir_ib_3m      = convert(dm.ir_ib_3m,'q');
dv.dl_czk_eur    = convert(dm.dl_czk_eur,'q');
dv.dl_oil        = convert(dm.dl_oil,'q');
dv.dl_food       = convert(dm.dl_food,'q');
dv.dl_export     = dq.dl_export;
dv.dl_gdp_eu     = dq.dl_gdp_eu;

% alternatives
% dv.dl_gdp_ez     = dq.dl_gdp_ez;
% dv.dl_oil = dq.dl_oil;
% dv.dl_food = dq.dl_food;
% dv.dl_gdp_us = dq.dl_gdp_us;
% dv.unempl_rate = dq.unempl_rate;
% dv.gdp_eu_gap = dq.gdp_eu_gap;

smpl     = qq(2002,4):get(dv.dl_y,'last'); % set estimation range 
smplf    = smpl(end)+(1:rngntf_q); % set forecast range
smplfull = smpl(1):smplf(end); % set full range

dummylist = {'dummy1'}; %define dummy variables

% not necessary
dv.dummy1 = tseries(smplfull,0);
dv.dummy1(qq(2008,3))  = 1;
% dv.dummy2 = tseries(smplfull,0);
% dv.dummy2(qq(2015,1))  = 1;

% order of VAR
p = 2;

% Set up Litterman priors for bayesian estimation of the VAR.
relstd   = nanstd(db2tseries(dv,list));
nobs     = length(smpl); 
dummyobs = BVAR.litterman(0,sqrt(nobs)*relstd,1);

% Define new Bayesian VAR with exogenous variables
ww = VAR(list,'exogenous',dummylist);

% Estimate BVAR
[w, data] = estimate(ww, dv, smpl, 'order=',p, 'bvar',dummyobs);

% compute forecast three months ahead
f = forecast(w, dv, smplf, dv, 'meanonly',true, 'dboverlay',true); 

for idx = list_rep
    % move projection into final database
    id = idx{:};
    df.(id) = f.(id); 

    % extend level by projected growth
    df.(id(2:end)) = dq.(id(2:end));
    for ii = smpl(end)+(1:rngntf_m)
     df.(id(2:end))(ii) = df.(id(2:end))(ii-1) + df.(id)(ii)/4;
    end

    % calculate original seasonally adjusted level
    df.(id(4:end)) = exp(df.(id(2:end))/100);
    
    % calculate seasonally unadjusted level 
    df.([id(4:end),'_sf']) = df.(id(4:end))/dq.([id(4:end),'_su']);
    df.([id(4:end),'_sf'])(end+(1:4)) = df.([id(4:end),'_sf'])(end+(-3:0));
    df.([id(4:end),'_su']) = df.(id(4:end))/df.([id(4:end),'_sf']);
end

% evaluate preformance of the BVAR model
if BvarperformanceReport
rep = report_var_performance(w, dv, smpl, p, rep, label);
end

rep = report_gdp_onepager(df, smpl(end), list_rep, rep, label);

% calculate RMSE for alternative specifications
rmse = sqrt(data.res_dl_y(:)'*data.res_dl_y(:));
% disp('RMSE of GDP QoQ Growth Rate')
% disp([rmse])

dbsave(dmf,'results/bvar_inflation.csv');

disp('Preparing the report...')

% Publish Report
rep.publish('results/NTF report.pdf', 'display', false);
disp('NTF DONE!!!')
rmpath('ntf');

end

%% preparing obs variables for the filter
% Data transformation
% FOR EXTERNAL VARIABLES, USE THE LATEST EXTERNAL ASSUMPTIONS FROM OGR

 %dq = dboverlay(dq,de);
 dObs = struct();
 
 %=== GDP level
 tmp = struct();
 dObs.OBS_L_GDP = 100*log(dq.gdp_10_sa);
  
 %=== Core CPI level
 tmp.cpi_net = convert(dm.cpi_core,'q');
 dObs.OBS_L_CPIXFE = 100*log(tmp.cpi_net);
  
 %=== Headline CPI level
 tmp.cpi = convert(x12(dm.cpi_su),'q');
 dObs.OBS_L_CPI = 100*log(tmp.cpi);
  
 %=== Inflation target
 dq.infl_tar_lo = convert(dm.infl_tar_lo,'q');
 dq.infl_tar_up = convert(dm.infl_tar_up,'q');
 dq.infl_tar = convert(dm.infl_tar,'q');
 
 tmp.infl_tar = [tseries(qq(1999,1), 5);(dq.infl_tar_lo+dq.infl_tar_up)/2; dq.infl_tar];
 dObs.OBS_D4L_CPI_TAR = 100*log(1+tmp.infl_tar/100);
  
 %=== Interest rate
 dObs.OBS_RS = convert(dm.ir_cb_repo,'q');
  
 %=== Exchange rate
 dObs.OBS_S = convert(dm.czk_usd,'q');
  
 tmp.czk_eur = convert(dm.czk_eur,'q');
 dObs.OBS_L_S_EUR_UNC = 100*log(tmp.czk_eur);
  
 %=== Unemployment rate
 tmp.unr = x12(dq.unempl_rate_su);
 dObs.OBS_UNR = tmp.unr;

  %% External  
  %=== US vars
 dObs.OBS_L_GDP_US     = de.OBS_L_GDP_US;%
 dObs.OBS_L_CPI_US     = de.OBS_L_CPI_US;%
 dObs.OBS_RS_US        = de.OBS_RS_US;%
 
 tmp.cpi_core_us     = x12(dm.cpi_core_us_su);
 tmp.OBS_L_CPIXFE_US = 100*log(convert(tmp.cpi_core_us,'q'));
 rngCpiCore = get(de.OBS_DLA_CPIFXE_US,'range');
 for ind = 1:numel(rngCpiCore)
     tmp.OBS_L_CPIXFE_US(rngCpiCore(ind)) = tmp.OBS_L_CPIXFE_US(rngCpiCore(ind)-1) + de.OBS_DLA_CPIFXE_US(rngCpiCore(ind))/4;
 end
 dObs.OBS_L_CPIXFE_US  = tmp.OBS_L_CPIXFE_US;
 
 dObs.OBS_RR_BAR_US    = de.OBS_RR_BAR_US;%
 %dObs.OBS_RR_BAR_US    = dq.OBS_RR_BAR_OGR_US;%
 
%  dObs.OBS_L_S_USD_EUR  = tseries();
%  dObs.OBS_L_S_USD_EUR(endhist+2:get(de.OBS_L_S_USD_EUR,'last'))  = 100*log(de.OBS_L_S_USD_EUR);%
  
  %=== EZ vars
 tmp.cpi_core_ez     = x12(dm.cpi_core_ez_su);
 dObs.OBS_L_CPIXFE_EZ   = 100*log(convert(tmp.cpi_core_ez,'q'));%
 %rngCpiCore = get(de.OBS_DLA_CPIXFE_EZ,'range');
 %for ind = 1:numel(rngCpiCore)
 %    dObs.OBS_L_CPIXFE_EZ(rngCpiCore(ind)) = dObs.OBS_L_CPIXFE_EZ(rngCpiCore(ind)-1) + de.OBS_DLA_CPIXFE_EZ(rngCpiCore(ind))/4;
 %end
 dObs.OBS_DLA_CPIXFE_EZ = de.OBS_DLA_CPIXFE_EZ;%
  
  %=== Food & oil
 dObs.OBS_L_OIL         = de.OBS_L_OIL;%
 dObs.OBS_L_FOOD        = de.OBS_L_FOOD;%
 dObs.OBS_DLA_QOIL_BAR  = de.OBS_DLA_QOIL_BAR;%
 dObs.OBS_DLA_QFOOD_BAR = de.OBS_DLA_QFOOD_BAR;%
  
  %=== Gaps
 dObs.OBS_L_GDP_GAP_US  = de.OBS_L_GDP_GAP_US;
 %dObs.OBS_L_GDP_GAP_US  = dObs.OBS_L_GDP_GAP_OGR_US;
 dObs.OBS_L_GDP_GAP_CH  = de.OBS_L_GDP_GAP_CH;
 dObs.OBS_L_GDP_GAP_EA7 = de.OBS_L_GDP_GAP_EA7;
 dObs.OBS_L_GDP_GAP_EU  = de.OBS_L_GDP_GAP_EU;
 %dObs.OBS_L_GDP_GAP_EU  = de.OBS_L_GDP_GAP_OGR_EU;
 dObs.OBS_L_GDP_GAP_JA  = de.OBS_L_GDP_GAP_JA;
 dObs.OBS_L_GDP_GAP_LA6 = de.OBS_L_GDP_GAP_LA6;
 dObs.OBS_L_GDP_GAP_RC6 = de.OBS_L_GDP_GAP_RC6;

 dObs.OBS_L_Z_GAP_CH    = de.OBS_L_Z_GAP_CH;  
 dObs.OBS_L_Z_GAP_EA7   = de.OBS_L_Z_GAP_EA7;
 dObs.OBS_L_Z_GAP_EU    = de.OBS_L_Z_GAP_EU;
 dObs.OBS_L_Z_GAP_JA    = de.OBS_L_Z_GAP_JA;
 dObs.OBS_L_Z_GAP_LA6   = de.OBS_L_Z_GAP_LA6;
 dObs.OBS_L_Z_GAP_RC6   = de.OBS_L_Z_GAP_RC6;

%% saving the results
save results/data.mat dObs starthist endhist;
save results/repdata.mat dm dq dy;

disp('Done.');

rmpath('data');
rmpath('results');
rmpath('utils');