% Housekeeping
clear all;
close all;
clc;

addpath('results');

% Load model, solve it, etc.
%% Load model 
try load model.mat
catch a1_readmodel
end

% Defining ranges
rng = qq(00,1):qq(00,4);
startfore = qq(01,1);
endfore = startfore + 24;
prng = rng(1):endfore;


% Display that the code is being processed
disp('Generating Impulse Response Functions...');

%% Generating IRFs for 1pp shocks from a given list

% Introduce a list of shocks we want to simulate
shock_list = {'SHK_DLA_CPIXFE', 'SHK_L_GDP_GAP', 'SHK_L_S','SHK_RS',...
              'SHK_DLA_Z_BAR','SHK_D4L_CPI_TAR','SHK_DLA_GDP_BAR',...
              'SHK_CTYPREM'};

%shock_name = get(m,'ecomments');
rep = report.new('IRFs Report - Deviations from steady-state',...
    'visible', false,'colsep',0.1);

%remove not important shocks
% rm_list = {'SHK_L_GDP_GAP_LA6','SHK_L_Z_GAP_RC6'};
rm_list = {};


num = [];
for ind = 1: length(rm_list)
    for j = 1: length(shock_list)
        if strcmp(shock_list{j},rm_list{ind})
            num=[num,j];
        end
    end
end

for ind = length(num):-1:1;
shock_list(num(ind)) = [];
end

% Introduce a loop across the list of shocks
for i = 1:numel(shock_list)
    if strcmp(shock_list{i},'SHK_D4L_CPI_TAR')
        m.rho_D4L_CPI_TAR = 1;
        m   = sstate(m, 'blocks=', true, 'growth=', true);
        m   = solve(m);
    end
    
disp([shock_list{i},' ...']);
%x = title(x,shock_name{i},'fontsize','Large');

% Create database of model variables with steady-state values
d = zerodb(m,rng);

% Introducing a 1pp shock in period 'startfore'
d.(shock_list{i})(startfore) = 1;

% Simulate 
s = simulate(m, d, startfore:endfore,'deviation',true);
s = dboverlay(s,d);

% Plotting results
f = figure('visible','of');
subplot(331);
    plot(prng, [s.DLA_CPIXFE s.D4L_CPI_TAR]);
    legend('Core','CPI tar','location','SE')
    title('Inflation (pp, QoQ)');
    grid on;
subplot(332);
    plot(prng, [s.DLA_S]);
    title('Nom. Exch. Rate Depr. (pp, QoQ @ar)');
    grid on;
subplot(333);
    plot(prng, [s.DLA_Z]);
    title('Real Exch. Rate Depr. (pp, QoQ @ar)');
    grid on;
subplot(334);
    plot(prng, [s.L_S, s.L_CPIXFE]);
    title('Nominal Exchange Rate and Core CPI (100*log)');
    legend('FX USD','Core','location','SE');
%     legend('Nominal Exchange Rate','Headline CPI Index','Location','Southoutside');
    grid on;
subplot(335);
    plot(prng, [s.RS]);
    title('Nominal Interest Rate (pp, p.a.)');
    grid on;
subplot(336);
    plot(prng, [s.RR_GAP]);
    title('Short-term real Int.Rate Gap (pp)');
    grid on;
subplot(337);
    plot(prng, [s.L_GDP_GAP]);
    title('Output Gap (pp)');
    grid on;
 subplot(338);
    plot(prng, [s.L_Z_GAP]);
    title('RER gap (pp)');
    grid on;
subplot(339);
    plot(prng, [s.L_REER_GAP]);
    title('REER gap (pp)');
    grid on;
rep.userfigure(shock_list{i},f);
end

%% compile output to .ps or .pdf file

rep.publish(['results' filesep 'irfs_report_' datestr(now,'yyyymmmdd_HHMMSS') '.pdf'], 'display', false);
disp('Done !!!');

rmpath('results');
