%%%%%%%%%%%%%%%%%%%%%%%%
%%% In-sample Simulation
%%%%%%%%%%%%%%%%%%%%%%%%
clear all
close all
clc

addpath('results');
addpath('utils');

%% Load model 
try load model.mat
catch a1_readmodel
end

%% Time span
rtime = qq(2003,1); %start of sample

stime = qq(2008,1);qq(2004,1); %starting point of the first simulation 
                    %stime is the initial state
                    %stime+1 is the first simulated time period
etime = qq(2016,1); %the end of the known history
       
%% Selection of historical time series for computing model's forecasting
%properties:
list_xnames = {'DLA_CPIXFE','D4L_CPIXFE','L_GDP_GAP','D4L_GDP','L_S','RS','RR_GAP',...
    'RR_BAR','L_Z_GAP','DLA_Z_BAR','L_REER_GAP','L_QOIL_GAP','L_QFOOD_GAP'};

list_titles = {'QoQ Core Inflation','YoY Coree Inflation','Output Gap (%)','YoY GDP Growth (%)',...
    'Nominal Exchange Rate (100*log)','Nominal Interest Rate (% pa)','Real Interest Rate Gap (%)'...
    'Real Interest Rate Tnd', 'RER (to US) Gap (%)','RER (to US) Tnd Growth (% QoQ @ar)','REER Gap (%)',...
    'l qoil gap','l qfood gap'};


%% Database preparation
% loading filter result, always run a3_kalman_filter
load filterdata.mat;

for i = 1:length(list_xnames)
    list_out{i} = ['out_' list_xnames{i}];
end

for i = 1:length(list_xnames)
    eval([list_out{i} '= resize(d.(list_xnames{i}), rtime:etime);'])
end

model = m;

%% Remove all residuals from the database
names   = get(model,'enames');
list_ = names;

for ind = 1:numel(names)
    d.(list_{ind}) = tseries(rtime:etime, zeros(size(rtime:etime)));
end

% list_ = names;
% list_db = dbnames(d);
% %rng = get(list_db(1),'first2last');
% 
% for i = 1:length(list_)
%     if strcmp(list_{i}, list_db{i})
%         d.(list_{i}) = tseries(rtime:etime, zeros(size(rtime:etime)));
%     else
%         disp(['Missing residual variable in database: ', list_{i}]);
%     end
% end

%% Simulations ... (done by a "loop")
%Beginning of the "loop" ...
t = 0;
for time = stime:1:etime
    fcastrng = time:time+8;
    if fcastrng(end)>etime
        exorng = time:etime;
    else
        exorng = time:time+8;
    end

disp(' ');
disp('---------------------------------------------------------------------')
disp(['The first simulated time period of this projection round: ',char(dat2str(time+1))]);
disp(' ');

% not endogenized:
% 'Real Output Gap Shock (pp)'                  SHK_L_GDP_GAP       
% 'Core Inflation Shock (pp)'                   SHK_DLA_CPIXFE
% 'Core/Headline RP Gap Shock (pp)'             SHK_L_RP_GAP
% 'Policy Shock (pp)'                           SHK_RS
% 'Nominal Exchange Rate Shock (pp)'            SHK_L_S
% 'Unemployment Rate Gap Shock (pp)'            SHK_UNR_GAP
% 'NAIRU Trend Shock (pp)'                      SHK_NAIRU
% 'NAIRU Trend Growth Shock (pp)'               SHK_D_NAIRU
% 'Real Exchange Rate Trend Shock (pp)'         SHK_L_Z_BAR  <--- not used in the model, just not deleted 



% endogenized:
% 'Core/Headline RP Trend Shock (pp)'           SHK_DLA_RP_BAR
% SHK_DLA_S_USD_EUR
% SHK_L_S_EUR_POL
% SHK_DLA_S_EUR_POL
% 'Headline Inflation Target Shock (pp)'        SHK_D4L_CPI_TAR
% 'Real Output Trend Growth Shock (pp)'         SHK_DLA_GDP_BAR
% 'Risk Premium Shock (pp)'                     SHK_CTYPREM
% 'Real Exchange Rate Trend Growth Shock (pp)'  SHK_DLA_Z_BAR

% 'US Real Output Trend Growth Shock (pp)'    SHK_DLA_GDP_BAR_US
% 'US Real Output Gap Shock (pp)'             SHK_L_GDP_GAP_US
% 'US Monetary Policy Shock (pp)'             SHK_RS_US
% 'US Real Interest Rate Trend Shock (pp)'    SHK_RR_BAR_US
% 'US Headline Inflation Shock (pp)'          SHK_DLA_CPI_US
% 'US Core Inflation Shock (pp)'              SHK_DLA_CPIXFE_US

% 'World Real Oil Trend Shock (pp)'            SHK_DLA_QOIL_BAR
% 'World Real Oil Gap Shock (pp)'              SHK_L_QOIL_GAP
% 'Anticipated World Real Oil Gap Shock (pp)'  SHKA_L_QOIL_GAP
% 'World Real Food Trend Shock (pp)'           SHK_DLA_QFOOD_BAR
% 'World Real Food Gap Shock (pp)'             SHK_L_QFOOD_GAP



% Plan for "targets", "foreign variables", etc ...
fcast_plan  = plan(model,fcastrng);
 
fcast_plan  = exogenize(fcast_plan, 'DLA_S_USD_EUR', exorng);
fcast_plan  = endogenize(fcast_plan, ' SHK_DLA_S_USD_EUR', exorng);

% fcast_plan  = exogenize(fcast_plan, 'DLA_S_EUR_POL',           exorng);
% fcast_plan  = endogenize(fcast_plan, 'SHK_DLA_S_EUR_POL',        exorng);

fcast_plan  = exogenize(fcast_plan, 'D4L_CPI_TAR',           exorng);
fcast_plan  = endogenize(fcast_plan, 'SHK_D4L_CPI_TAR',        exorng);

fcast_plan  = exogenize(fcast_plan, 'RR_BAR',           exorng);
fcast_plan  = endogenize(fcast_plan, 'SHK_RR_BAR',        exorng);

fcast_plan  = exogenize(fcast_plan, 'DLA_CPIXFE_US',           exorng);
fcast_plan  = endogenize(fcast_plan, 'SHK_DLA_CPIXFE_US',        exorng);

fcast_plan  = exogenize(fcast_plan, 'DLA_GDP_BAR_US',           exorng);
fcast_plan  = endogenize(fcast_plan, 'SHK_DLA_GDP_BAR_US',        exorng);

fcast_plan  = exogenize(fcast_plan, 'L_GDP_GAP_US',           exorng);
fcast_plan  = endogenize(fcast_plan, 'SHK_L_GDP_GAP_US',        exorng);

fcast_plan  = exogenize(fcast_plan, 'RS_US',           exorng);
fcast_plan  = endogenize(fcast_plan, 'SHK_RS_US',        exorng);

fcast_plan  = exogenize(fcast_plan, 'RR_BAR_US',           exorng);
fcast_plan  = endogenize(fcast_plan, 'SHK_RR_BAR_US',        exorng);

fcast_plan  = exogenize(fcast_plan, 'DLA_CPI_US',           exorng);
fcast_plan  = endogenize(fcast_plan, 'SHK_DLA_CPI_US',        exorng);

fcast_plan  = exogenize(fcast_plan, 'DLA_QOIL_BAR',           exorng);
fcast_plan  = endogenize(fcast_plan, 'SHK_DLA_QOIL_BAR',        exorng);

fcast_plan  = exogenize(fcast_plan, 'L_QOIL_GAP',           exorng);
fcast_plan  = endogenize(fcast_plan, 'SHK_L_QOIL_GAP',        exorng);

fcast_plan  = exogenize(fcast_plan, 'DLA_QFOOD_BAR',           exorng);
fcast_plan  = endogenize(fcast_plan, 'SHK_DLA_QFOOD_BAR',        exorng);

fcast_plan  = exogenize(fcast_plan, 'L_QFOOD_GAP',           exorng);
fcast_plan  = endogenize(fcast_plan, 'SHK_L_QFOOD_GAP',        exorng);

fcast_plan  = exogenize(fcast_plan, 'DLA_Z_BAR',           exorng);
fcast_plan  = endogenize(fcast_plan, 'SHK_DLA_Z_BAR',        exorng);

list_cc = {'EU','JA','CH','EA7','LA6','RC6'};

for cc = 1:length(list_cc)
    
fcast_plan  = exogenize(fcast_plan, ['L_GDP_GAP_' list_cc{cc}],           exorng);
fcast_plan  = endogenize(fcast_plan, ['SHK_L_GDP_GAP_' list_cc{cc}],        exorng);

fcast_plan  = exogenize(fcast_plan, ['L_Z_GAP_' list_cc{cc}],           exorng);
fcast_plan  = endogenize(fcast_plan, ['SHK_L_Z_GAP_' list_cc{cc}],        exorng);

end


%Simulate the model
dbfcast = simulate(model,d,fcastrng,'plan',fcast_plan,'anticipate',true,'nonlinear',12);
s = dboverlay(d,dbfcast);
    
%Store the results
%list = get(mmng,'xnames');

range_ = fcastrng(1)-1:fcastrng(end);

for i = 1:length(list_xnames)
    s.(list_xnames{i}) = resize(s.(list_xnames{i}),range_);
end

%s = dbbatch(s, '$0', 'resize(s.$0, range_)','namelist', list);
    
for i = 1:length(list_xnames)
    eval([ list_out{i} '= [' list_out{i} ', s.(list_xnames{i})];'])
end

eval(['s',num2str(t),'= s;']); 

t = t+1;
end %end of the simulation "loop"


% Evaluating out-of-sample performance
%% Compute forecast error statistics

d = dbcol(d,1);
d = dbclip(d,stime+get(m,'maxlag'):etime);

% remove everything except endogenous variables from the database
d = d*get(m,'xlist');

mea = [];
sta = [];
sse = [];
rmse = [];
rmse_rw = [];

% subtract baseline and in-sample simulations for each variable to get
% descriptive statistics (std of fcast errors, mean squared errors, ...)
disp('Computing descriptive statistics...')

nPer = etime - stime;
for j=1:length(list_xnames)
  
  di1=zeros(nPer+1,8);
  rw=zeros(nPer+1,8);
  for t=0:nPer
    u=eval(['s',num2str(t),'.',list_xnames{j}]);
    v=eval(['d.',list_xnames{j}]);
    % 1 period ahead
    di1(t+1) = u(stime+t)-v(stime+t);
    rw(t+1) = v(stime+t)-v(stime-1+t);
    % 2 periods ahead ...
    di1(t+1,2) = u(stime+1+t)-v(stime+1+t);
    rw(t+1,2) = v(stime+1+t)-v(stime-1+t);

    di1(t+1,3) = u(stime+2+t)-v(stime+2+t);
    rw(t+1,3) = v(stime+2+t)-v(stime-1+t);
    
    di1(t+1,4) = u(stime+3+t)-v(stime+3+t);
    rw(t+1,4) = v(stime+3+t)-v(stime-1+t);
    
    di1(t+1,5) = u(stime+4+t)-v(stime+4+t);
    rw(t+1,5) = v(stime+4+t)-v(stime-1+t);
    
    di1(t+1,6) = u(stime+5+t)-v(stime+5+t);
    rw(t+1,6) = v(stime+5+t)-v(stime-1+t);
    % ...7 periods ahead
    di1(t+1,7) = u(stime+6+t)-v(stime+6+t);
    rw(t+1,7) = v(stime+6+t)-v(stime-1+t);
    % 8 periods ahead
    di1(t+1,8) = u(stime+7+t)-v(stime+7+t);
    rw(t+1,8) = v(stime+7+t)-v(stime-1+t);
  end
  
  % print table
  fprintf('%15s \t\t\tmean,std and rmse\n',list_xnames{j});
   
  b  = nanmean(di1,1);
  b2 = nanstd(di1);
   
  di1(isnan(di1)) = 0;
  b3 = (sum(di1.^2))/size(di1,1) ;
  b4 = sqrt(b3);
  
  rw(isnan(rw)) = 0;
  b5=sqrt((sum(rw.^2))/size(rw,1));
  
  mea = [mea; b];
  sta = [sta; b2];
  sse = [sse; b3];
  rmse = [rmse; b4];
  rmse_rw = [rmse_rw; b5];
  
end


%% Graphs
disp('Creating report ...');

srep = qq(2003,1); % beginning of report
erep = etime+8; % end

rep = report.new('In-sample Simulations', 'dateformat', 'YY:P', 'visible', false);

for i = 1:length(list_xnames)
    f = figure('visible','off');
    eval([ 'p = plot(srep:erep,' list_out{i} ');'])
    title(list_titles{i});
    set(p(1),'linewidth',4);
    for j = 2:length(p)
        set(p(j),'linestyle','--','linewidth',2);
    end
    hold on
    grid
    rep.userfigure(list_xnames(i),f);
end
% RMSE
% for i = 1:length(list_xnames)
%     
%     eval(['[M, me, med, s, err2] = revival(' list_out{i} ', stime:etime, 6);']);
%     rownames_ = {'Mean','Median','Std.','Err2'};
%     for jj = 1:length(me)
%         colnames_{jj} = ['t+' num2str(jj)];
%     end
% 
%     rep = matrix(x, [me; med; s; err2],'text', list_titles {i}, ...
%                 'rownames', rownames_, 'colnames',colnames_,'decimal', 2);
% end

% if exist('In-sample.pdf','file') == 2
%     disp('CLOSE PDF READER AND CLICK ANY KEY WHEN DONE'); pause
% end

% prepare tables:
listqua={'1q','2q','3q','4q','5q','6q','7q','8q'};
for i=1:length(list_xnames)
  rowNames{i}=list_titles{i};
end
rowIndex=nan(length(list_xnames),1);
for i=1:length(list_xnames)
  rowIndex(i)=strmatch(list_xnames{i},list_xnames,'exact');
end

rep.pagebreak;
rep.matrix('Mean of error',mea(rowIndex,:),'rownames',rowNames,'colnames',listqua,'format','%.3f');
rep.pagebreak;
rep.matrix('Std of error',sta(rowIndex,:),'rownames',rowNames,'colnames',listqua,'format','%.3f');
rep.pagebreak;
rep.matrix('Mean squared error',sse(rowIndex,:),'rownames',rowNames, 'colnames',listqua,'format','%.3f');
rep.pagebreak;
rep.matrix('Root mean squared error',rmse(rowIndex,:),'rownames',rowNames, 'colnames',listqua,'format','%.2f');
rep.pagebreak;
rmseR=rmse./rmse_rw;
rep.matrix('RMSE comparison to RW',rmseR(rowIndex,:),'rownames',rowNames, 'colnames',listqua,'format','%.2f');


rep.publish(['results' filesep 'hist_sim_' datestr(now,'yyyymmmdd_HHMMSS') '.pdf'], 'display', false);
disp('Done !!!');

rmpath('results');