function [mNew, parNew, stdNew, mult] = estim_vcovmult(m, d, rng, std_list,std)

% estim_vcovmult: estimate multiplier of vcov for selected shocks
% by OGResearch

par = get(m,'parameters');

a   = 0.1;
b   = 10;
eps = 0.001;
N   = 100;

c  = (-1+sqrt(5))/2;
x1 = c*a + (1-c)*b;
x2 = (1-c)*a + c*b;

pp = dbbatch(std,'$0','x1*std.$0','namelist',std_list);
p = dbbatch(par,'$0','x1*par.$0','namelist',std_list);
m = assign(m, p);
m = sstate(m);
m = solve(m);
%[fx1,se2,F,pe,A,Pa,pedind,pred,smooth] = loglik(m,d,rng,'deviation',false,'std',pp,'relative',false);
fx1 = loglik(m,d,rng,'deviation',false,'std',pp,'relative',false);

pp = dbbatch(std,'$0','x2*std.$0','namelist',std_list);
p = dbbatch(par,'$0','x2*par.$0','namelist',std_list);
m = assign(m, p);
m = sstate(m);
m = solve(m);
%[fx2,se2,F,pe,A,Pa,pedind,pred,smooth] = loglik(m,d,rng,'deviation',false,'std',pp,'relative',false);
fx2 = loglik(m,d,rng,'deviation',false,'std',pp,'relative',false);

fprintf('------------------------------------------------------\n');
fprintf(' x1 x2 f(x1) f(x2) b - a\n');
fprintf('------------------------------------------------------\n');
fprintf('%.4e %.4e %.4e %.4e %.4e\n', x1, x2, fx1, fx2, b-a);
for i = 1:N-2
   if fx1 < fx2
      b = x2;
      x2 = x1;
      fx2 = fx1;
      x1 = c*a + (1-c)*b;
      pp = dbbatch(std,'$0','x1*std.$0','namelist',std_list);
      p = dbbatch(par,'$0','x1*par.$0','namelist',std_list);
      m = assign(m, p);
      m = sstate(m);
      m = solve(m);
      %[fx1,se2,F,pe,A,Pa,pedind,pred,smooth] = loglik(m,d,rng,'deviation',false,'std',pp,'relative',false);
      fx1 = loglik(m,d,rng,'deviation',false,'std',pp,'relative',false);
   else
      a = x1;
      x1 = x2;
      fx1 = fx2;
      x2 = (1-c)*a + c*b;
      pp=dbbatch(std,'$0','x2*std.$0','namelist',std_list);
      p=dbbatch(par,'$0','x2*par.$0','namelist',std_list);
      m = assign(m, p);
      m = sstate(m);
      m = solve(m);
      %[fx2,se2,F,pe,A,Pa,pedind,pred,smooth] = loglik(m,d,rng,'deviation',false,'std',pp,'relative',false);
      fx2 = loglik(m,d,rng,'deviation',false,'std',pp,'relative',false);
   end;
   fprintf('%.4e %.4e %.4e %.4e %.4e\n', x1, x2, fx1, fx2, b-a);
   if (abs(b-a) < eps)
      fprintf('succeeded after %d steps\n', i);
      mult = (a+b)/2;
      stdNew = dbbatch(std,'$0','mult*std.$0','namelist',std_list);
      parNew = dbbatch(par,'$0','mult*par.$0','namelist',std_list);
      mNew = assign(m,parNew);
      mNew = sstate(mNew);
      mNew = solve(mNew);
      stdvec = get(mNew, 'stdvec');
      enames = get(mNew, 'elist');
      for ii=1:numel(enames)
        stdvec(ii) = parNew.(['std_' enames{ii}]);
      end
      set(mNew, 'stdvec', stdvec);
      [mtmp,hist,se2,delta,pe] = filter(mNew, d, rng, stdNew, 'deviation', false, 'relative', false);
      report_likelihood_contrib(hist.mean,stdNew,rng,std_list);
      return;
  end;
end
fprintf('failed requirements after %d steps\n', N);

end

function report_likelihood_contrib(db, mult, rng, std_list)
  enames = regexprep(std_list,'^std_','');
  disp('Contributions of shocks to the overall minus log likelihood');
  for i=1:length(enames)
    s = db.(enames{i})(rng);
    st = mult.(['std_' enames{i}])(rng);
    disp(sprintf('%30s: %12.5g',enames{i},sum((s./st).^2)));
  end
end